# make installing with conda easier
# from kora import conda
# conda.install('packagename')

# first install miniconda like
# !wget -O mini.sh https://repo.anaconda.com/miniconda/Miniconda3-4.3.21-Linux-x86_64.sh 
# !chmod +x mini.sh
# !bash ./mini.sh -b -f -p /usr/local

# also add path
# import sys
# sys.path.insert(5, '/usr/local/lib/python3.6/site-packages')

# then install and add python==3.6 to allow it to work with colab
# !conda install -q -y --prefix /usr/local python=3.6 # add your_library
# can set channel as well inside {params}

import os, sys

os.system('wget -O mini.sh https://repo.anaconda.com/miniconda/Miniconda3-4.3.21-Linux-x86_64.sh')
os.chmod('mini.sh', 0o777)
os.system("./mini.sh -b -f -p /usr/local")
sys.path.insert(5, '/usr/local/lib/python3.6/site-packages')

def install(params):
    print("Use this command:")
    print(f'!conda install -q -y --prefix /usr/local {params} python=3.6')


# Change the behavior of google.colab.data_table to my liking
from IPython.display import Javascript, clear_output
from google.colab.output import eval_js

# First, 
# = %load_ext google.colab.data_table
get_ipython().run_line_magic('load_ext', 'google.colab.data_table')
# Then, make the default width to fit content, instead of 100%
from google.colab.data_table import DataTable
DataTable.min_width = '0'


def unload():
    """Unload the data_table extension"""
    get_ipython().run_line_magic('unload_ext', 'google.colab.data_table')


def filter_df(df):
    """Desplay the dataframe and let user select rows"""
    display(DataTable(df, min_width='300'))
    display(Javascript('''
      var bag = new Set([])
      function show_checkboxes(){
        var cells = document.querySelectorAll('td.index_column')
        for (c of cells) {
          var v = c.classList[2]  // "id-0"
          var cb = checkbox(v, bag.has(v))
          c.append(cb)
        }
        var nums = document.querySelector('.google-visualization-table-page-numbers')
        if (nums) nums.onclick = show_checkboxes  // ต้องดักใหม่ทุกครั้ง
      }
      function checkbox(value, checked=false) {
        cb = document.createElement('input')
        cb.type = 'checkbox'
        cb.value = value
        cb.checked = checked
        cb.onchange = update_bag
        return cb
      }
      function update_bag(){
        var cb = event.target
        if (cb.checked) {
          bag.add(cb.value)
        } else {
          bag.delete(cb.value)
        }
        console.log(bag)
      }
      function show_return_button(){
        var b = document.createElement('button')
        b.id = 'return'
        b.innerHTML = 'Return'
        document.querySelector('#output-body').append(b)
      }
      show_checkboxes()
      show_return_button()

      var result = new Promise(resolve =>{
        var button = document.getElementById('return')
        button.onclick = ()=>{
          resolve([...bag]) // convert to list
        }
      })
    '''))
    bag = eval_js('result')
    clear_output()
    ids = [int(v[3:]) for v in bag]
    return df.iloc[ids]


from os import *  

# modify os to my liking
from subprocess import Popen, PIPE, STDOUT
from time import sleep

def system(cmd):
    process = Popen(cmd, shell=True,
                    stdout=PIPE, stderr=STDOUT, 
                    universal_newlines=True)
    while True:
        output = process.stdout.readline()
        print(output, end='')
        sleep(0.1)
        if process.poll() is not None:
            # remaining output 
            for output in process.stdout.readlines():
                print(output, end='')
            break


# Next
# ====

# os.chmod also accept +x

# os.copy instead of shutil

# os.getoutput instead of subprocess


import json
from requests import models

from IPython.display import HTML

_render_template = """
<script src="https://rawgit.com/caldwell/renderjson/master/renderjson.js"></script>
<script>
renderjson.set_show_to_level(1)
document.body.appendChild(renderjson(%s))
new ResizeObserver(google.colab.output.resizeIframeToContent).observe(document.body)
</script>
"""

models.Response._repr_html_ = lambda rsp: _render_template % rsp.text


def render(jstr):
    if type(jstr) != str:
        jstr = json.dumps(jstr)
    return HTML(_render_template % jstr)


import json
from pathlib import Path
from subprocess import getoutput
from IPython.display import HTML

_TEMPLATE = '''
<script src="//code.jquery.com/jquery-3.4.1.min.js"></script>
<link href="//cdn.jsdelivr.net/npm/jquery.fancytree@2.27/dist/skin-win8/ui.fancytree.min.css" rel="stylesheet">
<script src="//cdn.jsdelivr.net/npm/jquery.fancytree@2.27/dist/jquery.fancytree-all-deps.min.js"></script>
<div id='tree'></div>
<script>
$("#tree").fancytree({
  source: %s
})
</script>
'''

def fancytree(data):
    """ Display data with fancytree.

    Data should be in a correct format, i.e.
    data = [
        {"title": "Node 1", "key": "1"},
        {"title": "Folder 2", "key": "2", "folder": True, "children": [
            {"title": "Node 2.1", "key": "3"},
            {"title": "Node 2.2", "key": "4"}
        ]}
    ]
    """ 
    return HTML(_TEMPLATE % json.dumps(data))


def convert(node):
    """ Convert a compact format into fancytree data format """
    data = []
    for k, v in node.items():
        d = {'title': k}
        if v:
            d['folder'] = True
            d['children'] = convert(v)
            d['title'] = f"{k} ({len(d['children'])})"
        data.append(d)
    return data


def path_tree(path='.'):
    """ Given a path to a directory, display its tree """
    root = {}
    for p in Path(path).rglob('*'):
        node = root
        for part in p.parts:
            if part not in node:
                node[part] = {}
            node = node[part]
    data = convert(root)
    return fancytree(data)


def tar_tree(filename):
    """ Display contents inside a tar file using fancytree """
    root = {}
    lines = getoutput('tar -tzf '+filename).splitlines()
    for line in lines:
        if line.endswith('/'): continue
        node = root
        for part in line.split('/'):
            if part not in node:
                node[part] = {}
            node = node[part]
    data = convert(root)
    return fancytree(data)




# install chromium, its driver, and selenium
import os
os.system('apt install chromium-chromedriver')
os.system('pip install selenium')
# set options to be headless, ..
from selenium import webdriver
options = webdriver.ChromeOptions()
options.add_argument('--headless')
options.add_argument('--no-sandbox')
options.add_argument('--disable-dev-shm-usage')
# create a webdriver instance, ready to use
wd = webdriver.Chrome('chromedriver',options=options)


from urllib.request import urlretrieve
from matplotlib import *  # font_manager, ft2font, rc


def add_thai_font():
  url = 'https://github.com/korakot/kora/raw/master/THSarabunChula.ttf'
  add_font(url)


def add_font(url):
  if url.startswith('http'):
    filename = urlretrieve(url)[0]  # in /tmp/...
  else:
    filename = url   # already manual download to colab
  name = get_font_name(filename)
  font_manager.fontManager.addfont(filename)
  set_font(family=name)     # select this font automatically


def get_font_name(filename):
  font = ft2font.FT2Font(filename)
  return font.family_name


def set_font(**kw):
  rc('font', **kw) # family, size


from google.colab.output import *
from IPython.display import display, Javascript

def set_height(h):
    eval_js('google.colab.output.setIframeHeight("%s")' % h)

# Usage:
# set_style(fontSize='30px', color='red')
# print("Hello")
def set_style(**kw):
    import json
    display(Javascript('''
    for (rule of document.styleSheets[0].cssRules){
        if (rule.selectorText=='body') {
            Object.assign(rule.style, %s)
            break
        }
    }
    ''' % json.dumps(kw)))


"""
Make using ffmpeg easier. Define functions instead of calling !ffmpeg
"""

__all__ = ['cut', 'probe', 'Video', 'sample_video']

import os
from subprocess import getoutput

from IPython.display import HTML

from .json import render


def cut(input='video.mp4', start=0.0, end=60.0, output=None):
    def ts(mn: float):
        m, s = divmod(100*mn, 100)
        return '{:02.0f}:{:05.2f}'.format(m,s)

    if output is None:
        ext = input.split('.')[-1]
        output = 'output.'+ext
    params = f'-i {input} -ss {ts(start)} -to {ts(end)} -codec copy {output}'
    os.system(f'ffmpeg {params}')
    return output
# Example: cut('x.m4a', 2.10, 2.30)


def probe(filename):
    cmd = f'ffprobe -v quiet -show_format -show_streams -print_format json {filename}'
    return render(getoutput(cmd))


def Video(filename, width=400):
    os.system(f'ln -snf {filename} /usr/local/share/jupyter/nbextensions/v.mp4')
    return HTML(f"<video width={width} src='/nbextensions/v.mp4' controls/>")


# a good test video in Colab
sample_video = '/usr/local/lib/python3.6/dist-packages/imageio/resources/images/realshort.mp4'